/*
 * Copyright (C) 2010 Bjoern Biesenbach <bjoern@bjoern-b.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <string.h>
#include <stdio.h>

#include "ds1371.h"
#include "rf12.h"

#include "DS18X20/onewire.h"
#include "DS18X20/ds18x20.h"

#define CHANNEL         1           // Sende/Empfangskanal (0-3) (nur gültig wenn kein DIP Schalter verwendet wird)
#define RF_BAUDRATE     20000       // Baudrate des RFM12 (nur gültig wenn kein DIP Schalter verwendet wird)
#define MODUL_ID 0

#define DS18X20_PULLUP_PIN PD6
#define DS1371_INT_PIN PD2
#define DS1371_WDS_PIN PD4

uint8_t gSensorID[OW_ROMCODE_SIZE];

void search_sensor(void)
{
    uint8_t i;
    uint8_t id[OW_ROMCODE_SIZE];
    uint8_t diff;
    
    
    DS18X20_find_sensor( &diff, &id[0] );
    if( diff == OW_PRESENCE_ERR ) {
        rf12_txpacket("no sensors found\r\n", 19, 0x02, 0);
    }

    if( diff == OW_DATA_ERR ) {
        rf12_txpacket("bus error\r\n", 8, 0x02, 0);
    }
    
    for (i=0;i<OW_ROMCODE_SIZE;i++)
        gSensorID[i]=id[i];
}

int main(void){

    char string_to_send[128];
    uint8_t subzero, cel_frac_bits,cel;
    uint8_t measuring = 0;

    GICR = (1<<INT0);
   
    PORTD = (1<<DS1371_INT_PIN);
    DDRD = (1<<DS1371_WDS_PIN) | (1<<DS18X20_PULLUP_PIN);
  

    ds1371_start_alarm_timer();
    ds1371_set_alarm_time(0);
    rf12_init(1);
    rf12_config(RF_BAUDRATE, CHANNEL, 0, QUIET);
    PORTD |= (1<<DS18X20_PULLUP_PIN);
    _delay_ms(10);
    search_sensor();
    PORTD &= ~(1<<DS18X20_PULLUP_PIN);
    sei();
    while(1){
        wdt_enable(WDTO_250MS);
        if(!measuring){
            PORTD |= (1<<DS18X20_PULLUP_PIN);
            _delay_ms(1);
            DS18X20_start_meas( DS18X20_POWER_PARASITE, &gSensorID[0] );
            measuring = 1;
            ds1371_set_alarm_time(1);
        }
        else{
            rf12_init(0);
            DS18X20_read_meas( &gSensorID[0], &subzero, &cel, &cel_frac_bits);
            if(subzero)
                sprintf(string_to_send,"%d;%d;-%d;%d;%d\r\n",MODUL_ID,1,cel,cel_frac_bits*625, 0);
            else
                sprintf(string_to_send,"%d;%d;%d;%d;%d\r\n",MODUL_ID,1,cel,cel_frac_bits*625, 0);
            rf12_txpacket(string_to_send, strlen(string_to_send), 0x02, 0);
            rf12_sleep();
            measuring = 0;
            PORTD &= ~(1<<DS18X20_PULLUP_PIN);
            ds1371_set_alarm_time(1);
        }
        wdt_disable();
        set_sleep_mode(SLEEP_MODE_PWR_DOWN);
        sleep_mode();
    }
}

ISR(SIG_INTERRUPT0){
    ds1371_clear_alarm_timer();
}


